package com.tanpu.community.manager;

import com.tanpu.common.api.CommonResp;
import com.tanpu.common.exception.BizException;
import com.tanpu.community.api.beans.qo.FormerThemeQo;
import com.tanpu.community.api.beans.qo.NotificationQo;
import com.tanpu.community.api.beans.qo.ThemeNotifyQo;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.req.notification.NotifyQueryReq;
import com.tanpu.community.api.beans.vo.feign.fatools.UserInfoResp;
import com.tanpu.community.api.enums.ThemeTypeEnum;
import com.tanpu.community.cache.RedisCache;
import com.tanpu.community.dao.entity.community.NotificationEntity;
import com.tanpu.community.dao.entity.community.TopicEntity;
import com.tanpu.community.feign.fatools.FeignClientForFatools;
import com.tanpu.community.service.BatchFeignCallService;
import com.tanpu.community.service.NotificationService;
import com.tanpu.community.service.ThemeService;
import com.tanpu.community.service.TopicService;
import com.tanpu.community.util.ConvertUtil;
import com.tanpu.community.util.TimeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

import static com.tanpu.community.api.constants.RedisKeyConstant.CACHE_FEIGN_USER_INFO;
import static com.tanpu.community.api.constants.RedisKeyConstant.CACHE_FORWARD_THEME_ID;

@Service
public class NotificationManager {

    @Autowired
    private NotificationService notificationService;

    @Autowired
    private TopicService topicService;

    @Autowired
    private ThemeService themeService;

    @Resource
    private BatchFeignCallService batchFeignCallService;

    @Autowired
    private FeignClientForFatools feignClientForFatools;

    @Autowired
    private RedisCache redisCache;

    public List<ThemeNotifyQo> queryList(NotifyQueryReq req, String userId){
        List<NotificationEntity> query = notificationService.query(userId, req.getType(), req.getLastId(),req.getPageSize());
        List<ThemeNotifyQo> themeNotifyQos = ConvertUtil.notificationEntitiy2ThemeQos(query);
        for (ThemeNotifyQo themeNotifyQo : themeNotifyQos) {

            // 用户信息
            UserInfoResp userInfo = redisCache.getObject(StringUtils.joinWith("_", CACHE_FEIGN_USER_INFO, themeNotifyQo.getAuthorId()),
                    60, () -> this.getUserInfo(themeNotifyQo.getAuthorId()), UserInfoResp.class);
                   themeNotifyQo.setAuthorId(userInfo.getUserId());
                   themeNotifyQo.setNickName(userInfo.getNickName());
                   themeNotifyQo.setUserImg(userInfo.getHeadImageUrl());
                   themeNotifyQo.setUserType(userInfo.getUserType());
                   themeNotifyQo.setLevelGrade(userInfo.getLevelGrade());
                   themeNotifyQo.setUserInvestorType(userInfo.getUserInvestorType());
                   themeNotifyQo.setBelongUserOrgId(userInfo.getBelongUserOrgId());
                   themeNotifyQo.setBelongUserOrgName(userInfo.getBelongUserOrgName());
            // 封装转发对象
            if (StringUtils.isNotEmpty(themeNotifyQo.getFormerThemeId())){
                FormerThemeQo former = redisCache.getObject(StringUtils.joinWith("_", CACHE_FORWARD_THEME_ID, themeNotifyQo.getFormerThemeId()), 60,
                        () -> this.getFormerTheme(themeNotifyQo.getFormerThemeId()), FormerThemeQo.class);
                if (former!=null){
                    themeNotifyQo.setFormerUserName(former.getNickName());
                    if (ThemeTypeEnum.DISCUSSION.getCode().equals(former.getThemeType())){
                        themeNotifyQo.setContent(former.getContent().get(0).getValue());
                    }
                    if (ThemeTypeEnum.LONG_TEXT.getCode().equals(former.getThemeType())){
                        themeNotifyQo.setContent(former.getTitle());
                    }

                }

            }
            if (StringUtils.isNotEmpty(themeNotifyQo.getTopicId())){
                TopicEntity topicEntity = topicService.queryById(themeNotifyQo.getTopicId());
                themeNotifyQo.setTopicTitle(topicEntity.getTopicTitle());
            }
        }
        return themeNotifyQos;
    }


    private UserInfoResp getUserInfo(String authorId) {
        CommonResp<UserInfoResp> userInfoNewCommonResp = feignClientForFatools.queryUserInfoNew(authorId);
        if (userInfoNewCommonResp.isNotSuccess()) {
            throw new BizException("内部接口调用失败");
        }
        return userInfoNewCommonResp.getData();
    }


    public NotificationQo queryBriefInfo(String userId){

        return  NotificationQo.builder().message(" {用户名}关注了你").updateCount(99).updateTime(TimeUtils.format(LocalDateTime.now())).build();


    }

    //返回被转发主题
    private FormerThemeQo getFormerTheme(String formerThemeId) {
        if (StringUtils.isNotBlank(formerThemeId)) {
            ThemeQo formerTheme = ConvertUtil.themeEntityToQo(themeService.queryByThemeId(formerThemeId));
            if (formerTheme != null) {
                batchFeignCallService.getAttachDetail(formerTheme);
                FormerThemeQo f = ConvertUtil.themeQo2FormerThemeQo(formerTheme);
                return f;
            }

        }
        return null;
    }
}
